var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as React from 'react';
import { createElement, instantiate } from 'govern';
/**
 * A factory to create a `<Store to>` React element, with typing.
 * @param store The store to subscribe to
 * @param children A function to render each of the store's values.
 */
export function createStore(of, children) {
    return React.createElement(Store, { of: of, children: children });
}
/**
 * Accepts an observable as a prop, and passes each of its values to the
 * component's children via a render function.
 *
 * NOTE: this component should be generic on output type, but TSX doesn't
 * properly support generic components. In the meantime, if you want
 * typing, use the `createSubscribe` factory.
 *
 * See https://github.com/Microsoft/TypeScript/issues/14729.
 */
var Store = /** @class */ (function (_super) {
    __extends(Store, _super);
    function Store(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.state = {};
        _this.isDispatching = false;
        _this.store = instantiate(createElement(Flatten, { children: _this.props.of }));
        return _this;
    }
    Store.prototype.componentWillMount = function () {
        if (!this.props.of) {
            console.warn("An \"element\" prop must be provided to <Store> but \"" + this.props.of + "\" was received.");
        }
    };
    Store.prototype.componentWillReceiveProps = function (nextProps) {
        if (!nextProps.of) {
            console.warn("A \"element\" prop must be provided to <StoreProps> but \"" + this.props.of + "\" was received.");
        }
        // As elements are immutable, we can skip a lot of updates by
        // checking if the `to` element/store has changed.
        if (nextProps.of !== this.props.of) {
            this.store.setProps({
                children: nextProps.of,
            });
        }
    };
    Store.prototype.componentDidCatch = function (error) {
        this.disposeStore();
    };
    Store.prototype.componentWillUnmount = function () {
        this.disposeStore();
    };
    Store.prototype.disposeStore = function () {
        if (this.store) {
            this.store.dispose();
            delete this.store;
        }
    };
    Store.prototype.render = function () {
        return this.props.children(this.store);
    };
    return Store;
}(React.Component));
export { Store };
/**
 * A Govern component that accepts an element or store, instantiates and
 * updates props if required, and returns the output.
 */
function Flatten(props) {
    return props.children;
}
//# sourceMappingURL=Store.js.map