var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
import * as React from 'react';
import * as PropTypes from 'prop-types';
import { createElement, instantiate } from 'govern';
/**
 * A factory to create a `<Subscribe to>` React element, with typing.
 * @param store The store to subscribe to
 * @param children A function to render each of the store's values.
 */
export function createSubscribe(store, children) {
    return React.createElement(Subscribe, { to: store, children: children });
}
/**
 * Accepts an observable as a prop, and passes each of its values to the
 * component's children via a render function.
 *
 * NOTE: this component should be generic on output type, but TSX doesn't
 * properly support generic components. In the meantime, if you want
 * typing, use the `createSubscribe` factory.
 *
 * See https://github.com/Microsoft/TypeScript/issues/14729.
 */
var Subscribe = /** @class */ (function (_super) {
    __extends(Subscribe, _super);
    function Subscribe(props, context) {
        var _this = _super.call(this, props, context) || this;
        _this.receiveError = function (error) {
            // Grab errors from the store, and throw them so a higher level React
            // component can handle it with componentDidCatch.
            throw error;
        };
        _this.handleChange = function (output, dispatch) {
            _this.isAwaitingRenderFromFlush = true;
            _this.isAwaitingRenderFromProps = false;
            _this.setState({ output: output, dispatch: dispatch });
        };
        _this.handleStartDispatch = function () {
            _this.isDispatching = true;
        };
        _this.handleEndDispatch = function () {
            _this.isDispatching = false;
            if (_this.isAwaitingRenderFromProps) {
                // Now that `isDispatching` is false, `shouldComponentUpdate` will return
                // true, so we can flush through our changes by setting a dummy object.
                _this.setState({ dummy: {} });
            }
        };
        _this.state = {};
        _this.isDispatching = false;
        _this.isAwaitingRenderFromProps = false;
        if (context.govern_priority) {
            _this.priority = context.govern_priority;
        }
        else {
            _this.priority = 1;
        }
        return _this;
    }
    Subscribe.prototype.getChildContext = function () {
        return {
            govern_priority: this.priority + 1,
        };
    };
    Subscribe.prototype.componentWillMount = function () {
        if (!this.props.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + this.props.to + "\" was received.");
        }
        this.store = instantiate(createElement(Flatten, { children: this.props.to }));
        this.handleChange(this.store.getValue(), this.store.UNSAFE_dispatch);
        // Create subsription within `componentWillMount` instead of in
        // `constructor`, as it is possible for components to have side effects
        // on initial subscription (as the first `componentDidFlush` is held
        // until then, and it can cause `setState`.
        this.subscription = this.store.subscribe(this.handleChange, this.receiveError, undefined, this.handleStartDispatch, this.handleEndDispatch, String(this.priority));
    };
    Subscribe.prototype.componentWillReceiveProps = function (nextProps) {
        if (!nextProps.to) {
            console.warn("A \"to\" prop must be provided to <Subscribe> but \"" + this.props.to + "\" was received.");
        }
        // If no flush is received during dispatch, we'll want to re-render
        // manually when it completes.
        if (this.isDispatching) {
            this.isAwaitingRenderFromProps = true;
        }
        // As elements are immutable, we can skip a lot of updates by
        // checking if the `to` element/store has changed.
        if (nextProps.to !== this.props.to) {
            this.store.setProps({
                children: nextProps.to,
            });
        }
    };
    Subscribe.prototype.componentDidCatch = function (error) {
        this.cleanup();
    };
    Subscribe.prototype.componentWillUnmount = function () {
        this.cleanup();
    };
    Subscribe.prototype.cleanup = function () {
        if (this.store) {
            this.subscription.unsubscribe();
            this.store.dispose();
            delete this.subscription;
            delete this.store;
        }
    };
    Subscribe.prototype.shouldComponentUpdate = function () {
        return !this.isDispatching || this.isAwaitingRenderFromFlush;
    };
    Subscribe.prototype.render = function () {
        this.isAwaitingRenderFromFlush = false;
        this.isAwaitingRenderFromProps = false;
        return this.props.children(this.state.output, this.state.dispatch);
    };
    Subscribe.contextTypes = {
        govern_priority: PropTypes.number,
    };
    Subscribe.childContextTypes = {
        govern_priority: PropTypes.number,
    };
    return Subscribe;
}(React.Component));
export { Subscribe };
/**
 * A Govern component that accepts an element or store, instantiates and
 * updates props if required, and returns the output.
 */
function Flatten(props) {
    return props.children;
}
//# sourceMappingURL=Subscribe.js.map